package main

import (
	"encoding/base64"
	"encoding/binary"
	"encoding/json"
	"fmt"
	"io/ioutil"
	"log"
	"math/rand"
	"os"
	"strconv"

	"srs.epita.fr/fic-server/libfic"
)

type ResponsesUpload struct {
	Keys map[string]string `json:"flags"`
	MCQs map[int64]bool    `json:"mcqs"`
}

func treatSubmission(pathname string, team fic.Team, exercice_id string) {
	bid := make([]byte, 5)
	binary.LittleEndian.PutUint32(bid, rand.Uint32())
	id := "[" + base64.StdEncoding.EncodeToString(bid) + "]"
	log.Println(id, "New submission receive", pathname)

	var responses ResponsesUpload

	if cnt_raw, err := ioutil.ReadFile(pathname); err != nil {
		log.Println(id, "[ERR]", err)
	} else if err := json.Unmarshal(cnt_raw, &responses); err != nil {
		log.Println(id, "[ERR]", err)
	} else if eid, err := strconv.Atoi(exercice_id); err != nil {
		log.Println(id, "[ERR]", err)
	} else if exercice, err := fic.GetExercice(int64(eid)); err != nil {
		log.Println(id, "[ERR]", err)
	} else if theme, err := exercice.GetTheme(); err != nil {
		log.Println(id, "[ERR]", err)
	} else if s, tm := team.HasSolved(exercice); s {
		log.Printf("%s [WRN] Team %d ALREADY solved exercice %d (%s : %s)\n", id, team.Id, exercice.Id, theme.Name, exercice.Title)
	} else {
		if solved, err := exercice.CheckResponse(responses.Keys, responses.MCQs, team); err != nil {
			log.Println(id, "[ERR]", err)
			genTeamMyFile(team)
		} else if solved {
			exercice.Solved(team)
			log.Printf("%s Team %d SOLVED exercice %d (%s : %s)\n", id, team.Id, exercice.Id, theme.Name, exercice.Title)
			if err := os.Remove(pathname); err != nil {
				log.Println(id, "[ERR]", err)
			}

			// Write event
			if lvl, err := exercice.GetLevel(); err != nil {
				log.Println(id, "[ERR]", err)
			} else if _, err := fic.NewEvent(fmt.Sprintf("L'équipe %s a résolu le <strong>%d<sup>e</sup></strong> challenge %s !", team.Name, lvl, theme.Name), "alert-success"); err != nil {
				log.Println(id, "[WRN] Unable to create event:", err)
			}
			genTeamAll(team)
		} else {
			log.Printf("%s Team %d submit an invalid solution for exercice %d (%s : %s)\n", id, team.Id, exercice.Id, theme.Name, exercice.Title)
			if err := os.Remove(pathname); err != nil {
				log.Println(id, "[ERR]", err)
			}

			// Write event (only on first try)
			if tm.Unix() == 0 {
				if lvl, err := exercice.GetLevel(); err != nil {
					log.Println(id, "[ERR]", err)
				} else if _, err := fic.NewEvent(fmt.Sprintf("L'équipe %s tente le <strong>%d<sup>e</sup></strong> challenge %s !", team.Name, lvl, theme.Name), "alert-warning"); err != nil {
					log.Println(id, "[WRN] Unable to create event:", err)
				}
			}
			genTeamMyFile(team)
		}
		if err := genEventsFile(); err != nil {
			log.Println("events.json generation error: ", err)
		}
	}
}
